<?php
/**
 * A class for directory reading and getting its contents
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007 - 2009 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkDirectoryPointer extends BaseFrameworkSystem {
	/**
	 * The current path we are working in
	 */
	private $pathName = '';

	/**
	 * The directory pointer
	 */
	private $dirPointer = null;

	/**
	 * Protected constructor
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Clean-up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Destructor for cleaning purposes, etc
	 */
	public function __destruct() {
		// Is there a resource pointer? Then we have to close the directory here!
		if (is_resource($this->getPointer())) {
			// Try to close a directory
			$this->closeDirectory();
		}

		// Call the parent destructor
		parent::__destruct();
	}

	/**
	 * Create a directory pointer based on the given path. The path will also
	 * be verified here.
	 *
	 * @param		$pathName					The path name we shall pass
	 *									to opendir()
	 * @param		$inConstructor				If we are in de/con-structor
	 *									or from somewhere else
	 * @throws	PathIsEmptyException		If the provided path name
	 *									is empty
	 * @throws	InvalidPathStringException	If the provided path name is
	 *									not a string
	 * @throws	PathIsNoDirectoryException	If the provided path name is
	 *									not valid
	 * @throws	PathReadProtectedException	If the provided path name is
	 *									read-protected
	 * @throws	DirPointerNotOpened			If opendir() returns not a
	 *									directory resource
	 * @return	$pointerInstance			A prepared instance of
	 *									FrameworkDirectoryPointer
	 */
	public final static function createFrameworkDirectoryPointer ($pathName, $inConstructor = false) {
		// Some pre-sanity checks...
		if (is_null($pathName)) {
			// No pathname given
			if ($inConstructor) {
				return null;
			} else {
				throw new PathIsEmptyException(null, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
			}
		} elseif (!is_string($pathName)) {
			// Is not a string
			if ($inConstructor) {
				return null;
			} else {
				throw new InvalidPathStringException(null, self::EXCEPTION_INVALID_STRING);
			}
		} elseif (!is_dir($pathName)) {
			// Not a directory
			if ($inConstructor) {
				return null;
			} else {
				throw new PathIsNoDirectoryException($pathName, self::EXCEPTION_INVALID_PATH_NAME);
			}
		} elseif (!is_readable($pathName)) {
			// Not readable
			if ($inConstructor) {
				return null;
			} else {
				throw new PathReadProtectedException($pathName, self::EXCEPTION_READ_PROTECED_PATH);
			}
		}

		// Try to open a handler
		$dirPointer = @opendir($pathName);
		if (!is_resource($dirPointer)) {
			// Something bad happend
			if ($inConstructor) {
				return null;
			} else {
				throw new DirPointerNotOpenedException($pathName, self::EXCEPTION_DIR_POINTER_INVALID);
			}
		}

		// Create new instance
		$pointerInstance = new FrameworkDirectoryPointer();

		// Set directory pointer and path name
		$pointerInstance->setPointer($dirPointer);
		$pointerInstance->setPathName($pathName);

		// Return the instance
		return $pointerInstance;
	}

	/**
	 * Read raw lines of data from a directory pointer and return the data
	 *
	 * @return	string	Directory and/or file names read from the current
	 *					directory pointer
	 */
	public function readRawDirectory () {
		// Read data from the directory pointer and return it
		return readdir($this->getPointer());
	}

	/**
	 * Read lines from the current directory pointer except some parts
	 *
	 * @param		$except	Some parts of a directory we want to ignore.
	 *					Valid: dirs
	 *					Other values will be silently ignored
	 * @return	string	Directory and/or file names read from the current
	 *					directory pointer
	 */
	public function readDirectoryExcept ($except = '') {
		if ((empty($except)) || (!is_array($except)) || (count($except) == 0)) {
			// No exception given, so read all data
			return $this->readRawDirectory();
		}

		// Read a raw line...
		$rawLine = $this->readRawDirectory();

		// Shall we exclude directories?
		if ((!is_null($rawLine)) && ($rawLine !== false) && (in_array($rawLine, $except))) {
			// Exclude this part
			return $this->readDirectoryExcept($except);
		} elseif ((!is_null($rawLine)) && ($rawLine !== false)) {
			// Return read data
			return $rawLine;
		}

		// End pointer reached
		return null;
	}

	/**
	 * Close a directory source and set it's instance to null and the path name
	 * to empty
	 *
	 * @return	void
	 */
	public function closeDirectory () {
		// Close the directory pointer and reset the instance variable
		@closedir($this->getPointer());
		$this->setPointer(null);
		$this->setPathName('');
	}

	/**
	 * Setter for the directory pointer
	 *
	 * @param		$dirPointer	The directory resource
	 * @return	void
	 */
	public final function setPointer ($dirPointer) {
		// Sanity-check if pointer is a valid directory resource
		if (is_resource($dirPointer) || is_null($dirPointer)) {
			// Is a valid resource
			$this->dirPointer = $dirPointer;
		} else {
			// Throw exception
			throw new InvalidDirectoryResourceException($this, self::EXCEPTION_INVALID_DIRECTORY_POINTER);
		}
	}

	/**
	 * Getter for the directory pointer
	 *
	 * @return	$dirPointer	The directory pointer which shall be a valid
	 *						directory resource
	 */
	public final function getPointer () {
		return $this->dirPointer;
	}

	/**
	 * Setter for path name
	 *
	 * @param		$pathName		The new path name
	 * @return	void
	 */
	public final function setPathName ($pathName) {
		$pathName = (string) $pathName;
		$this->pathName = $pathName;
	}

	/**
	 * Getter for path name
	 *
	 * @return	$pathName		The current path name
	 */
	public final function getPathName () {
		return $this->pathName;
	}
}

// [EOF]
?>
