<?php
/**
 * A standard random number generator
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007 - 2009 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class RandomNumberGenerator extends BaseFrameworkSystem {
	/**
	 * Prime number for better pseudo random numbers
	 */
	private $prime = 0;

	/**
	 * Add this calculated number to the rng
	 */
	private $extraNumber = 0;

	/**
	 * Extra salt for secured hashing
	 */
	private $extraSalt = '';

	/**
	 * Fixed salt for secured hashing
	 */
	private $fixedSalt = '';

	/**
	 * Maximum length for random string
	 */
	private $rndStrLen = 0;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of this class
	 * @return	void
	 */
	protected function __construct ($className = __CLASS__) {
		// Call parent constructor
		parent::__construct($className);

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$extraInstance	An extra instance for more salt (default: null)
	 * @return	$rngInstance	An instance of this random number generator
	 */
	public final static function createRandomNumberGenerator (FrameworkInterface $extraInstance = null) {
		// Get a new instance
		$rngInstance = new RandomNumberGenerator();

		// Initialize the RNG now
		$rngInstance->initRng($extraInstance);

		// Return the instance
		return $rngInstance;
	}

	/**
	 * Initializes the random number generator
	 *
	 * @param	$extraInstance	An extra instance for more salt (default: null)
	 * @return	void
	 * @todo	Add site key for stronger salt!
	 */
	protected function initRng ($extraInstance) {
		// Get the prime number from config
		$this->prime = $this->getConfigInstance()->readConfig('math_prime');

		// Calculate the extra number which is always the same unless you give
		// a better prime number
		$this->extraNumber = ($this->prime * $this->prime / (pi() ^ 2));

		// Seed mt_rand()
		mt_srand((double) sqrt(microtime() * 100000000 * $this->extraNumber));

		// Set the server IP to cluster
		$serverIp = "cluster";

		// Do we have a single server?
		if ($this->getConfigInstance()->readConfig('is_single_server') == 'Y') {
			// Then use that IP for extra security
			$serverIp = getenv('SERVER_ADDR');
		} // END - if

		// Yet-another fixed salt. This is not dependend on server software or date
		if ($extraInstance instanceof FrameworkInterface) {
			// With extra instance information
			$this->fixedSalt = sha1($serverIp . ':' . $extraInstance->__toString() . ':' . serialize($this->getDatabaseInstance()->getConnectionData()));
		} else {
			// Without extra information
			$this->fixedSalt = sha1($serverIp . ':' . serialize($this->getDatabaseInstance()->getConnectionData()));
		}

		// One-way data we need for "extra-salting" the random number
		$this->extraSalt = sha1($this->fixedSalt . ':' . getenv('SERVER_SOFTWARE') . ':' . $this->getConfigInstance()->readConfig('date_key') . $this->getConfigInstance()->readConfig('base_url'));

		// Get config entry for max salt length
		$this->rndStrLen = $this->getConfigInstance()->readConfig('rnd_str_length');
	}

	/**
	 * Makes a pseudo-random string useable for salts
	 *
	 * @param	$length			Length of the string, default: 128
	 * @return	$randomString	The pseudo-random string
	 */
	public function randomString ($length = -1) {
		// Is the number <1, then fix it to default length
		if ($length < 1) $length = $this->rndStrLen;

		// Initialize the string
		$randomString = '';

		// And generate it
		for ($idx = 0; $idx < $length; $idx++) {
			// Add a random character and add it to our string
			$randomString .= chr($this->randomNumber(0, 255));
		}

		// Return the random string a little mixed up
		return str_shuffle($randomString);
	}

	/**
	 * Generate a pseudo-random integer number in a given range
	 *
	 * @param	$min	Min value to generate
	 * @param	$max	Max value to generate
	 * @return	$num	Pseudo-random number
	 * @todo	I had a better random number generator here but now it is somewhere lost :(
	 */
	public function randomNumber ($min, $max) {
		return mt_rand($min, $max);
	}

	/**
	 * Getter for extra salt
	 *
	 * @return	$extraSalt
	 */
	public final function getExtraSalt () {
		return $this->extraSalt;
	}

	/**
	 * Getter for fixed salt
	 *
	 * @return	$fixedSalt
	 */
	public final function getFixedSalt () {
		return $this->fixedSalt;
	}
}

// [EOF]
?>
